// -*- C++ -*-
#include "Rivet/Analysis.hh"
#include "Rivet/Projections/ChargedFinalState.hh"
#include "Rivet/Tools/Logging.hh"

int Nevent1 = 0;//valid events
int control = 1;
double part_in_region = 0;
double maxpT = 0;
const double Area = 1.6755;
//int print = 1;


namespace Rivet {

  /// @brief CDF underlying event in leading track events
  /// @author Orestes Tumbarell Aranda (Havana), Hannes Jung (DESY)
  ///
  /// Rick Field's measurement of the underlying event in "leading track" events.
  ///
  /// @par Run conditions
  /// @arg \f$ \sqrt{s} = \f$ 300, 900, 1960 GeV
  /// @arg Run with generic QCD events.
  /// @arg Set particles with c*tau > 10 mm stable

  class CDF_2015_I1388868 : public Analysis {
  public:

    /// @name Constructors etc.
    //@{

    /// Constructor
    CDF_2015_I1388868()
      : Analysis("CDF_2015_I1388868")
    {    }

    //@}


  public:

    /// @name Analysis methods
    //@{

    /// Book histograms and initialise projections before the run
    void init() {
      
      const ChargedFinalState cfslead(-0.8, 0.8, 0.5*GeV);

      addProjection(cfslead, "CFSlead");
      
      //Energy selection
      if (fuzzyEquals(sqrtS()/GeV, 300, 1E-3)) 
         {
         control = 3;  
         cout << " CDF Tevatron UE: running with " << sqrtS()/GeV << endl;
         } 
         else if (fuzzyEquals(sqrtS()/GeV, 900, 1E-3)) 
         {
         control = 2; 
         cout << " CDF Tevatron UE: running with " << sqrtS()/GeV << endl;
         }
         else if (fuzzyEquals(sqrtS()/GeV, 1960, 1E-3)) 
         {
         control = 1; 
         cout << " CDF Tevatron UE: running with " << sqrtS()/GeV << endl;
         }
         else {
         cout << " not correct beam energy used " << sqrtS()/GeV << endl;
	   throw Error("Unexpected sqrtS ! Only 300, 900, 1960 GeV is supported");
      }
      
      
      //Preparing to divide histos
 
      _NchgPDFden1 = bookProfile1D(control,1,1);
      _NchgPMNden1 = bookProfile1D(control,1,2);
      _NchgPMXden1 = bookProfile1D(control,1,3);
      _NchgPden1 = bookProfile1D(control,1,4); 
             
      _PTsumPDFden1 = bookProfile1D(control,1,6); 
      _PTsumPMNden1 = bookProfile1D(control,1,7);
      _PTsumPMXden1 = bookProfile1D(control,1,8); 
      _PTsumPden1 = bookProfile1D(control,1,9);                                                                                                  											    
    }


    /// Perform the per-event analysis
    void analyze(const Event& event) {
      const double weight = event.weight();

      // Require at least one track in the event with pT >= 0.5 GeV
      const ChargedFinalState& cfslead = applyProjection<ChargedFinalState>(event, "CFSlead");
      if (cfslead.size() < 1) 
        vetoEvent;
      else	
	  Nevent1 = Nevent1 + 1;//valid_events
      
    
      // These are the charged particles (tracks) with pT > 1 GeV
      const ChargedFinalState& chargedlead = applyProjection<ChargedFinalState>(event, "CFSlead");
      
      // Identify leading track and its phi and pT (this is the same for both the 100 MeV and 500 MeV track cuts)
      ParticleVector particleslead = chargedlead.particlesByPt();
      
      
      Particle p_lead = particleslead[0];//largest pT per event
      const double philead = p_lead.momentum().phi();
      //const double etalead = p_lead.momentum().eta();
      const double PTmax  = p_lead.momentum().pT(); //perp
      
      
      double cont = 0;
      double NchgP1 = 0;// # c.particles por region
      double NchgP2 = 0;//"
      double PTsumP1 = 0;//scalar sum pT por region
      double PTsumP2 = 0;//"
       
      double NchgPtot = 0;
      double NchgPmax = 0;
      double NchgPmin = 0;
      double PTsumPtot = 0;
      double PTsumPmax = 0;
      double PTsumPmin = 0;
      
      double NchgPMXden = 0;
      double NchgPMNden = 0;
      double PTsumPMXden = 0;
      double PTsumPMNden = 0;

      double NchgPDFden=0;
      double PTsumPDFden=0;
      
      foreach (const Particle& p, particleslead) 
      {  //cfslead.particles()
        const double phi = p.momentum().phi();
        double newphi = 0;
        //const double signdphi = (philead - phi)/abs(philead - phi);//1 or -1
        const double pt = p.momentum().pT();
      
        //region definition
        if (phi > philead) newphi = phi - philead;
        else if (phi < philead) newphi = 2*PI - philead + phi;
        else newphi = 0;

        if ( (newphi > PI/3 && newphi < 2*PI/3) || (newphi > 4*PI/3 && newphi < 5*PI/3)) // transverse regions
        {
            part_in_region = part_in_region + weight;
	         
            if (PTmax > maxpT) maxpT = PTmax;
                      
            cont = cont + 1;
                 
            if (newphi > PI/3 && newphi < 2*PI/3)//transverse region 1
              {
              NchgP1 = NchgP1 + 1;
              PTsumP1 = PTsumP1 + pt;
              }
      
              else if (newphi > 4*PI/3 && newphi < 5*PI/3)//transverse region 2
              {
              NchgP2 = NchgP2 + 1;
              PTsumP2 = PTsumP2 + pt;
            } 
	
            NchgPtot = (NchgP1 + NchgP2)/2;
            NchgPmax = max(NchgP1,NchgP2);
            NchgPmin = min(NchgP1,NchgP2);
            PTsumPtot = (PTsumP1 + PTsumP2)/2;
            PTsumPmax = max(PTsumP1,PTsumP2);
            PTsumPmin = min(PTsumP1,PTsumP2);

	    PTsumPMXden = PTsumPmax/Area;
            PTsumPMNden = PTsumPmin/Area;
	    NchgPMXden = NchgPmax/Area;
            NchgPMNden = NchgPmin/Area;

            NchgPDFden = NchgPMXden - NchgPMNden;
            PTsumPDFden = PTsumPMXden - PTsumPMNden;              
        }
     
      }
     
      
      _NchgPden1->fill(PTmax / GeV ,  NchgPtot / ( Area ) , weight );
      _NchgPMXden1->fill(PTmax / GeV ,  NchgPmax / ( Area ) , weight );
      _NchgPMNden1->fill(PTmax / GeV ,  NchgPmin / ( Area ) , weight );
      _NchgPDFden1->fill(PTmax / GeV ,  NchgPDFden , weight );
      
      _PTsumPden1->fill(PTmax / GeV ,  PTsumPtot / ( Area ) , weight );
      _PTsumPMXden1->fill(PTmax / GeV ,  PTsumPmax / ( Area ) , weight );
      _PTsumPMNden1->fill(PTmax / GeV ,  PTsumPmin / ( Area ) , weight );
      _PTsumPDFden1->fill(PTmax / GeV ,  PTsumPDFden , weight );
      
    }


    /// Normalise histograms etc., after the run
    void finalize() {       
       
       cout << "CDF_2015_I1388868 UE: part_in_region = " << part_in_region << endl; 
       cout << "CDF_2015_I1388868 UE: valid_event = " << Nevent1 <<  endl;
       
       cout << "CDF_2015_I1388868 UE: maxpT = " << maxpT << endl;	
       
    }

    //@}


  private:

        /// @name Histograms
    //@{

    Profile1DPtr _NchgPden1, _NchgPMXden1,_NchgPMNden1,_NchgPDFden1,_PTsumPden1,_PTsumPMXden1,_PTsumPMNden1,_PTsumPDFden1;



  };



  // The hook for the plugin system
  DECLARE_RIVET_PLUGIN(CDF_2015_I1388868);

}
